#include <xc.h>
#include <plib.h>
#include "hw.h"

volatile int impulse_count;

//#pragma config UPLLEN   = OFF        // USB PLL
#pragma config FPLLMUL  = MUL_16        // PLL Multiplier
//#pragma config UPLLIDIV = DIV_1         // USB PLL Input Divider
#pragma config FPLLIDIV = DIV_2         // PLL Input Divider
#pragma config FPLLODIV = DIV_4         // PLL Output Divider
#pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
#pragma config FWDTEN   = OFF           // Watchdog Timer
#pragma config WDTPS    = PS1           // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
#pragma config OSCIOFNC = OFF           // CLKO Enable
#pragma config POSCMOD  = OFF       // Primary Oscillator
#pragma config IESO     = OFF           // Internal/External Switch-over
#pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = FRCPLL        // Oscillator Selection
#pragma config CP       = OFF           // Code Protect
#pragma config BWP      = OFF           // Boot Flash Write Protect
#pragma config PWP      = OFF           // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx1      // ICE/ICD Comm Channel Select
#pragma config DEBUG    = OFF            // Background Debugger Enable
#pragma config JTAGEN    = OFF            // Background Debugger Enable

volatile unsigned int dummy;

void init_mcu (void)
{
OSCCONbits.FRCDIV = 0;

DISP_RST_T = 0;
DISP_CS_T = 0;
DISP_CD_T = 0;
KPS_T = 0;
KCK_T = 0;
KOUT_T  = 1;
EK1_T = 1;
EK2_T = 1;
EK3_T = 1;
EK4_T = 1;
LED_S_T = 0;
LED_C_T = 0;
ELDAT_T = 0;
ELCK_T = 0;
CSA2_T = 0;
CSA1_T = 0;
CSA1 = 1;
CSA2 = 1;
//SYSTEMConfigPerformance(4000000L);


	PR3 = 50 * (PB_CLK / 32 / 1000);
	T3CONbits.TCKPS = 0b101;
	T3CONbits.TON = 1;
	IFS0bits.T3IF = 0;
	IEC0bits.T3IE = 1;	
	IPC3bits.T3IP = 2;
/*
	PR2 = 50 * (PB_CLK / 1000000);
	T2CONbits.TCKPS = 0b00;
	T2CONbits.TON = 1;
	IFS0bits.T2IF = 0;
	IEC0bits.T2IE = 1;	
	IPC2bits.T2IP = 2;
*/

CNENAbits.CNIEA8 = 1;
CNCONAbits.ON = 1;
IFS1bits.CNAIF = 0;
IEC1bits.CNAIE = 1;    // Enable ISR
IPC8bits.CNIP = 2;
IPC8bits.CNIS = 1;
INTEnableSystemMultiVectoredInt();


I2C1CON = 0x8000;
I2C1BRG = 0x7F;

ANSELA = 0x0001;
ANSELB = 0x8000;
ANSELC = 0x0000;


AD1CON1bits.SSRC = 0b111;
AD1CON1bits.ASAM = 1;
AD1CHS = 0;
AD1CON3bits.ADCS = 16;
AD1CON3bits.SAMC = 20;
AD1CON1SET = 0x8000;
IFS0bits.AD1IF = 0;	
IEC0bits.AD1IE = 1;	
IPC5bits.AD1IP = 2;

SPI1CONbits.MSTEN = 1;
SPI1CONbits.CKP = 0;
SPI1CONbits.SMP = 0;
SPI1CONbits.CKE = 1;
SPI1BRG = 1;
SPI1CONbits.ON = 1;

/*
 * MOSI - RPB11 - 0b0011
 * SCK - RPB14 - no remap
 * MISO - RPA1 - SDI1-0b0000
 *
 */

RPB11Rbits.RPB11R = 0b0011;
SDI1Rbits.SDI1R = 0b0000;

}



void iic_start (void)
{
I2C1CONbits.SEN = 1;
while (I2C1CONbits.SEN == 1);
}

void iic_restart (void)
{
I2C1CONbits.RSEN = 1;
while (I2C1CONbits.RSEN == 1);
}


void iic_stop (void)
{
I2C1CONbits.PEN = 1;
while (I2C1CONbits.PEN == 1);
}

void iic_write (unsigned char data)
{
I2C1TRN = data;
}

unsigned char iic_read (unsigned char ack_bit)
{
unsigned char tmp;
I2C1CONbits.RCEN = 1;
while (I2C1CONbits.RCEN == 1);
tmp = I2C1RCV;

if (ack_bit==0)
    I2C1CONbits.ACKDT = 0;
else
    I2C1CONbits.ACKDT = 1;
I2C1CONbits.ACKEN = 1;
while (I2C1CONbits.ACKEN == 1);
return tmp;
}

unsigned char write_spi (unsigned char data)
{
SPI1BUF = data;
while (SPI1STATbits.SPIBUSY==1);
return (SPI1BUF);
}


void disp_send(unsigned char cmd, unsigned char type)
{
if (type!=0)
	DISP_CD = 1;
else
	DISP_CD = 0;

	DISP_CS = 0;
	write_spi(cmd);
	DISP_CS = 1;
}

__inline void LCD_CtrlWrite_IC(unsigned char c)
{
	DISP_CD = 0;
	DISP_CS = 0;
	write_spi(c);
	DISP_CS = 1;
}

void  LCD_DataWrite_IC(unsigned char c)
{
	DISP_CD = 1;
	DISP_CS = 0;
	write_spi(c);
	DISP_CS = 1;
}

void disp_reset (void)
{
DISP_RST = 1;
dly_ms(10);
DISP_RST = 0;
dly_ms(10);
DISP_RST = 1;
dly_ms(10);
}


void dly_ms (unsigned int del)
{
	unsigned int i;
	for (i=0;i<del;i++)
		wait_1ms();
}

void wait_1ms (void)
{
T1CONbits.TON = 1;
PR1 = PB_CLK/8000;
TMR1=0;
while (IFS0bits.T1IF==0);
IFS0bits.T1IF=0;
}

void delay_cyc(unsigned int number)
{
IFS0bits.T1IF=0;
TMR1 = 0;
PR1 = number;
IFS0bits.T1IF = 0;
T1CONbits.TON = 1;
while (IFS0bits.T1IF==0);
IFS0bits.T1IF=0;
}


unsigned long read_keys (void)
{
unsigned char i;
unsigned long temp;
temp = 0;
KPS = 1;
delay_cyc(50);
KPS = 0;
for (i=0;i<8;i++)
	{
	temp = temp << 1;	
	if (KOUT == 0) temp = temp | 0x01;
	KCK = 1;
	delay_cyc(50);
	KCK = 0;
	delay_cyc(50);
	}
temp = temp >> 1;
temp = temp & 0x1F;
if (EK1==0) temp = temp|0x020;
if (EK2==0) temp = temp|0x040;
if (EK3==0) temp = temp|0x080;
if (EK4==0) temp = temp|0x100;
return temp;
}

void set_exp_local (unsigned int data)
{
unsigned char i;
unsigned long temp;
data = data << 7;
for (i=0;i<16;i++)	
	{
	if (data&0x01)
		LED_S = 1;
	else
		LED_S = 0;
	delay_cyc(50);
	LED_C = 1;
	data = data >> 1;
	delay_cyc(50);
	LED_C = 0;
	delay_cyc(50);
	}
}

void set_exp_rem (unsigned int data)
{
unsigned char i;
unsigned long temp;
for (i=0;i<8;i++)	
	{
	if (data&0x01)
		ELDAT = 1;
	else
		ELDAT = 0;
	delay_cyc(50);
	ELCK = 1;
	delay_cyc(50);
	data = data >> 1;
	ELCK = 0;
	delay_cyc(50);
	}
}

void iic_write_reg (unsigned char hw_addr, unsigned char addr, unsigned char data)
{
iic_start();
iic_write(hw_addr);
iic_write(addr);
iic_write(data);
iic_stop();
}

unsigned char iic_read_reg (unsigned char hw_addr, unsigned char addr)
{
unsigned char temp;
iic_start();
iic_write(hw_addr);
iic_write(addr);
iic_restart();
iic_write(hw_addr+1);
temp = iic_read(1);
iic_stop();
return temp;
}

/*
unsigned char sens_pres_get (unsigned char * state, unsigned long * pressure, char * temperature)
{
float alt;
unsigned char stat, ndat,th,tl,pu,ph,pl;
unsigned long pres;
alt = 0;
ndat = 0;
stat = *state;
if (stat==SENS_PRES_START)
    {
    iic_write_reg(0xC0, 0x26, 0x3A);
    stat=SENS_PRES_WAIT;
    }
else if (stat==SENS_PRES_WAIT)
{
pl = iic_read_reg(0xC0,0x26);
if ((pl&0x02)==0)
    {
    th = iic_read_reg(0xC0,0x04);
    tl = iic_read_reg(0xC0,0x05);
    pu = iic_read_reg(0xC0,0x01);
    ph = iic_read_reg(0xC0,0x02);
    pl = iic_read_reg(0xC0,0x03);
    pres = (((unsigned long)(pl))>>4)|(((unsigned long)(ph))<<4)|(((unsigned long)(pu))<<12);
    pres = pres / 4;
//    pres = get_avg_pres(pres);
    stat=SENS_PRES_START;
    *pressure = pres;
    *temperature = th;
    ndat = 1;
    }
}
else stat=SENS_PRES_START;
*state= stat;
return ndat;
}
*/

void init_press_sens (void)
{
iic_start();
iic_write(0xC0);
iic_write(0x26);
iic_write(0x21);
iic_stop();
dly_ms(100);
}

unsigned int measure_pres (void)
{
unsigned char d1,d2,d3;
unsigned long t;
int ret;
iic_start();
iic_write(0xC0);
iic_write(0x01);
iic_restart();
iic_write(0xC1);
d1 = iic_read(0);
d2 = iic_read(0);
d3 = iic_read(1);
iic_stop();
t = (((unsigned long)(d3))>>4)|(((unsigned long)(d2))<<4)|(((unsigned long)(d1))<<12);
t = t/4;
t = t/10;
ret = (int) (t);
return ret;
}


unsigned char measure_hum (void)
{
unsigned char d1,d2;
unsigned long t1;
char ret;
iic_start();
iic_write(0x4E);
iic_stop();
dly_ms(50);
	
iic_start();
iic_write(0x4F);
d1 = iic_read(0);
d2 = iic_read(1);
iic_stop();
dly_ms(1);

t1 = ((unsigned long)(d1))<<8 | d2;
t1 = t1*100;
t1 = t1/16383;
ret = (char)(t1);
return ret;
}

void init_temp_sens (void)
{
iic_start();
iic_write(0x90);
iic_write(0x01);
iic_write(0x00);
iic_stop();
}

int measure_temp (void)
{
unsigned char d1,d2;
unsigned int t;
int ret;
iic_start();
iic_write(0x90);
iic_write(0x05);
iic_stop();
dly_ms(50);
iic_start();
iic_write(0x91);
d1 = iic_read(0);
d2 = iic_read(1);
iic_stop();	
return d1;
}



void __ISR(_CHANGE_NOTICE_VECTOR, ipl2) ChangeNoticeHandler(void)
{
dummy = PORTA;
IFS1bits.CNAIF = 0;
if (PORTAbits.RA8 == 0)
	impulse_count++;

}


unsigned int get_gm_count (void)
{
unsigned int tmp;
tmp = impulse_count;
impulse_count = 0;
return tmp;
}

void __ISR(_TIMER_3_VECTOR, ipl2) Timer3Handler(void)
{
IFS0bits.T3IF = 0;	
timer_50ms();

}

volatile unsigned int adc_tmr;
volatile unsigned char adc_done;
volatile int adc_rets[ADC_LEN];


void adc_start (void)
{
adc_tmr = 0;
adc_done = 0;
AD1CON1SET = 0x8000;
}

unsigned char get_adc_state (void)
{
volatile unsigned char done;
done = 0;
IEC0bits.AD1IE = 0;
if (adc_done!=0) done = 1;
IEC0bits.AD1IE = 1;
return done;
}

void ADC_IRQHandler(void)
{
	/*
	if (adc_tmr<ADC_LEN)
		adc_rets[adc_tmr++] = ((LPC_ADC->DR[2] & 0xFFC0) >> 6);
	else
	{
		adc_done = 1;
		NVIC_DisableIRQ(ADC_IRQn);
	}
set_disp_pin(LED,0);
*/
}

volatile unsigned int a,tmr_count,tmp;
void __ISR(_TIMER_2_VECTOR, ipl2) Timer2Handler(void)
{
IFS0bits.T2IF = 0;
//AD1CON1SET = 0x0002;
}

void __ISR(_ADC_VECTOR, ipl2) ADCHandler(void)
{
IFS0bits.AD1IF = 0;	
	if (adc_tmr<ADC_LEN)
		adc_rets[adc_tmr++] = ADC1BUF0;
	else
	{
		adc_done = 1;
		AD1CON1CLR = 0x8000;
	}
}

void set_gain_chnl (unsigned char chnl, unsigned char gain_A, unsigned char gain_B)
{
CSA1 = 0;
write_spi(0x41);
write_spi(chnl);
CSA1 = 1;

dly_ms(5);

CSA1 = 0;
write_spi(0x40);
write_spi(gain_A);
CSA1 = 1;

dly_ms(5);

CSA2 = 0;
write_spi(0x40);
write_spi(gain_B);
CSA2 = 1;

}
